@@ -1,10 +1,16 @@
-hypervolume <- function(data, repsperpoint=1000, bandwidth, quantile=0.0, name=NULL, verbose=T, warnings=T)
+hypervolume <- function(data, repsperpoint=NULL, bandwidth, quantile=0.0, name=NULL, verbose=T, warnings=T)
 {
   data <- as.data.frame(data)
   
   dim = ncol(data)
   np = nrow(data)
   
+  if (is.null(repsperpoint))
+  {
+    repsperpoint = floor(100*10^sqrt(ncol(data)))
+    cat(sprintf('Choosing repsperpoint=%.0f (use a larger value for more accuracy.)\n',repsperpoint))
+  }
+  
   if (nrow(data) == 0)
   {
     stop('Hypervolume cannot be computed with empty input data.')
@@ -25,6 +31,11 @@ hypervolume <- function(data, repsperpoint=1000, bandwidth, quantile=0.0, name=N
     stop('Input bandwidth vector not same length as dimensionality of dataset.')
   }
   
+  if (any(bandwidth==0))
+  {
+    stop('Bandwidth must be non-zero.')
+  }
+  
   ### CHECK FOR WARNINGS
   if (warnings==TRUE)
   {
